@@ -68,18 +68,22 @@ const EOL: &str = "\n";
 
 impl Lyric {
     // GetText will fetch lyric by time in seconds
-    pub fn get_text(&self, time: i64) -> Option<String> {
+    pub fn get_text(&self, mut time: u64) -> Option<String> {
         if self.unsynced_captions.is_empty() {
             return None;
         };
 
         // here we want to show lyric 2 second earlier
-        let time = time * 1000 + 2000 + self.offset;
+        if self.offset > 0 {
+            time = time * 1000 + 2000 + self.offset.abs() as u64;
+        } else {
+            time = time * 1000 + 2000 - self.offset.abs() as u64;
+        }
 
-        let mut text = self.unsynced_captions.get(0)?.text.to_owned();
-        for v in self.unsynced_captions.iter() {
-            if time >= v.time_stamp as i64 {
-                text = v.text.to_owned();
+        let mut text = self.unsynced_captions.get(0)?.text.clone();
+        for v in &self.unsynced_captions {
+            if time >= v.time_stamp {
+                text = v.text.clone();
             } else {
                 break;
             }
@@ -87,17 +91,21 @@ impl Lyric {
         Some(text)
     }
 
-    pub fn get_index(&self, time: i64) -> Option<usize> {
+    pub fn get_index(&self, mut time: u64) -> Option<usize> {
         if self.unsynced_captions.is_empty() {
             return None;
         };
 
         // here we want to show lyric 1 second earlier
-        let time = time * 1000 + 2000 + self.offset;
+        if self.offset >= 0 {
+            time = time * 1000 + 2000 + self.offset.abs() as u64;
+        } else {
+            time = time * 1000 + 2000 - self.offset.abs() as u64;
+        }
 
         let mut index: usize = 0;
         for (i, v) in self.unsynced_captions.iter().enumerate() {
-            if time >= v.time_stamp as i64 {
+            if time >= v.time_stamp {
                 index = i;
             } else {
                 break;
@@ -106,7 +114,7 @@ impl Lyric {
         Some(index)
     }
 
-    pub fn adjust_offset(&mut self, time: i64, offset: i64) {
+    pub fn adjust_offset(&mut self, time: u64, offset: i64) {
         if let Some(index) = self.get_index(time) {
             // when time stamp is less than 10 seconds or index is before the first line, we adjust
             // the offset.
@@ -115,7 +123,11 @@ impl Lyric {
             } else {
                 // fine tuning each line after 10 seconds
                 let mut v = &mut self.unsynced_captions[index];
-                let adjusted_time_stamp = v.time_stamp as i64 + offset;
+                let adjusted_time_stamp = if offset > 0 {
+                    v.time_stamp + offset.abs() as u64
+                } else {
+                    v.time_stamp - offset.abs() as u64
+                };
                 v.time_stamp = match adjusted_time_stamp.cmp(&0) {
                     Ordering::Greater | Ordering::Equal => adjusted_time_stamp as u64,
                     Ordering::Less => 0,
@@ -127,21 +139,21 @@ impl Lyric {
             .sort_by(|b, a| b.time_stamp.cmp(&a.time_stamp));
     }
 
-    pub fn as_lrc_text(&mut self) -> Option<String> {
+    pub fn as_lrc_text(&self) -> String {
         let mut result: String = String::new();
         if self.offset != 0 {
             let string_offset = format!("[offset:{}]\n", self.offset);
             result += string_offset.as_ref();
         }
 
-        for line in self.unsynced_captions.iter() {
+        for line in &self.unsynced_captions {
             result += line.as_lrc().as_str();
         }
-        Some(result)
+        result
     }
 
     pub fn merge_adjacent(&mut self) {
-        let mut unsynced_captions = self.unsynced_captions.to_owned();
+        let mut unsynced_captions = self.unsynced_captions.clone();
         let mut offset = 1;
         for (i, v) in self.unsynced_captions.iter().enumerate() {
             if i < 1 {
@@ -166,7 +178,7 @@ impl UnsyncedCaption {
         //[00:12.00]Line 1 lyrics
         // !line.starts_with('[') | !line.contains(']')
         // First, parse the time
-        let time_stamp = UnsyncedCaption::parse_time(
+        let time_stamp = Self::parse_time(
             line.get(line.find('[').ok_or(())? + 1..line.find(']').ok_or(())?)
                 .ok_or(())?,
         )?;
@@ -176,6 +188,7 @@ impl UnsyncedCaption {
         Ok(Self { time_stamp, text })
     }
 
+    #[allow(clippy::cast_possible_truncation, clippy::cast_sign_loss)]
     fn parse_time(string: &str) -> Result<u64, ()> {
         //mm:ss.xx or mm:ss.xxx
         if !(string.contains(':')) | !(string.contains('.')) {
@@ -191,7 +204,8 @@ impl UnsyncedCaption {
         let micros = &format!("0.{}", string.get(y + 1..).ok_or(())?)
             .parse::<f64>()
             .map_err(|_| ())?;
-        let sum_milis = minute as u64 * 60 * 1000 + second as u64 * 1000 + (micros * 1000.0) as u64;
+        let sum_milis =
+            u64::from(minute) * 60 * 1000 + u64::from(second) * 1000 + (micros * 1000.0) as u64;
         Ok(sum_milis)
     }
 
@@ -257,7 +271,7 @@ impl FromStr for Lyric {
         // we sort the captions by Timestamp. This is to fix some lyrics downloaded are not sorted
         unsynced_captions.sort_by(|b, a| b.time_stamp.cmp(&a.time_stamp));
 
-        let mut lyric = Lyric {
+        let mut lyric = Self {
             offset,
             lang_extension,
             unsynced_captions,
