@@ -2,9 +2,6 @@
 //!
 //! `main_activity` is the module which implements the Main activity, which is the activity to
 //! work on termusic app
-
-mod playlist;
-mod queue;
 /**
  * MIT License
  *
@@ -31,11 +28,11 @@ mod queue;
 // Submodules
 // mod actions;
 // mod config;
+mod playlist;
+mod queue;
 mod update;
 mod view;
 mod youtube_options;
-
-// Locals
 use super::{Activity, Context, ExitReason};
 use crate::{
     config::{Termusic, MUSIC_DIR},
@@ -43,18 +40,17 @@ use crate::{
     song::Song,
     ui::activity::tageditor::TagEditorActivity,
 };
-use std::str::FromStr;
-// Ext
 use crossterm::terminal::{disable_raw_mode, enable_raw_mode};
 use log::error;
 use std::collections::VecDeque;
 use std::path::{Path, PathBuf};
+use std::str::FromStr;
 use std::sync::mpsc::{self, Receiver, Sender};
 use std::thread::sleep;
 use std::time::Duration;
 use tui_realm_treeview::Tree;
 use tuirealm::{Payload, Value, View};
-use youtube_options::{YoutubeOptions, YoutubeSearchState};
+use youtube_options::YoutubeOptions;
 
 // -- components
 const COMPONENT_LABEL_HELP: &str = "LABEL_HELP";
@@ -69,6 +65,8 @@ const COMPONENT_TEXT_ERROR: &str = "TEXT_ERROR";
 const COMPONENT_CONFIRMATION_RADIO: &str = "CONFIRMATION_RADIO";
 const COMPONENT_CONFIRMATION_INPUT: &str = "CONFIRMATION_INPUT";
 const COMPONENT_TEXT_MESSAGE: &str = "TEXT_MESSAGE";
+const COMPONENT_SEARCH_PLAYLIST_TABLE: &str = "SEARCH_PLAYLIST_LIST";
+const COMPONENT_SEARCH_PLAYLIST_INPUT: &str = "SEARCH_PLAYLIST_INPUT";
 
 /// ### `ViewLayout`
 ///
@@ -88,24 +86,15 @@ pub struct TermusicActivity {
     time_pos: u64,
     pub status: Option<Status>,
     pub current_song: Option<Song>,
-    sender: Sender<TransferState>,
-    receiver: Receiver<TransferState>,
+    sender: Sender<UpdateComponents>,
+    receiver: Receiver<UpdateComponents>,
     yanked_node_id: Option<String>,
     config: Termusic,
     youtube_options: YoutubeOptions,
-    sender_message: Sender<MessageState>,
-    receiver_message: Receiver<MessageState>,
-    sender_youtubesearch: Sender<YoutubeSearchState>,
-    receiver_youtubesearch: Receiver<YoutubeSearchState>,
     sender_queueitems: Sender<VecDeque<Song>>,
     receiver_queueitems: Receiver<VecDeque<Song>>,
 }
 
-pub enum MessageState {
-    Show((String, String)),
-    Hide,
-}
-
 #[derive(Clone, Copy)]
 pub enum Status {
     Running,
@@ -114,12 +103,16 @@ pub enum Status {
 }
 
 // TransferState is used to describe the status of download
-pub enum TransferState {
-    Running, // indicates progress
-    Success,
-    Completed(Option<String>),
-    ErrDownload,
-    ErrEmbedData,
+pub enum UpdateComponents {
+    DownloadRunning, // indicates progress
+    DownloadSuccess,
+    DownloadCompleted(Option<String>),
+    DownloadErrDownload,
+    DownloadErrEmbedData,
+    MessageShow((String, String)),
+    MessageHide((String, String)),
+    YoutubeSearchSuccess(YoutubeOptions),
+    YoutubeSearchFail(String),
 }
 
 // StatusLine shows the status of download
@@ -141,11 +134,8 @@ impl Default for TermusicActivity {
 
         let full_path = shellexpand::tilde(MUSIC_DIR);
         let p: &Path = Path::new(full_path.as_ref());
-        let (tx, rx): (Sender<TransferState>, Receiver<TransferState>) = mpsc::channel();
-        let (tx2, rx2): (Sender<MessageState>, Receiver<MessageState>) = mpsc::channel();
-        let (tx3, rx3): (Sender<YoutubeSearchState>, Receiver<YoutubeSearchState>) =
-            mpsc::channel();
-        let (tx4, rx4): (Sender<VecDeque<Song>>, Receiver<VecDeque<Song>>) = mpsc::channel();
+        let (tx, rx): (Sender<UpdateComponents>, Receiver<UpdateComponents>) = mpsc::channel();
+        let (tx2, rx2): (Sender<VecDeque<Song>>, Receiver<VecDeque<Song>>) = mpsc::channel();
         Self {
             exit_reason: None,
             context: None,
@@ -163,12 +153,8 @@ impl Default for TermusicActivity {
             yanked_node_id: None,
             config: Termusic::default(),
             youtube_options: YoutubeOptions::new(),
-            sender_message: tx2,
-            receiver_message: rx2,
-            sender_youtubesearch: tx3,
-            receiver_youtubesearch: rx3,
-            sender_queueitems: tx4,
-            receiver_queueitems: rx4,
+            sender_queueitems: tx2,
+            receiver_queueitems: rx2,
         }
     }
 }
@@ -184,7 +170,6 @@ impl TermusicActivity {
     pub fn run(&mut self) {
         match self.status {
             Some(Status::Stopped) => {
-                // self.update_queue_items();
                 if self.queue_items.is_empty() {
                     return;
                 }
