@@ -25,11 +25,11 @@
 use std::str::FromStr;
 // ext
 use super::{
-    youtube_options::YoutubeSearchState, ExitReason, MessageState, Status, StatusLine,
-    TermusicActivity, TransferState, COMPONENT_CONFIRMATION_INPUT, COMPONENT_CONFIRMATION_RADIO,
-    COMPONENT_INPUT_URL, COMPONENT_LABEL_HELP, COMPONENT_PARAGRAPH_LYRIC, COMPONENT_PROGRESS,
-    COMPONENT_TABLE_QUEUE, COMPONENT_TABLE_YOUTUBE, COMPONENT_TEXT_ERROR, COMPONENT_TEXT_HELP,
-    COMPONENT_TREEVIEW,
+    ExitReason, Status, StatusLine, TermusicActivity, UpdateComponents,
+    COMPONENT_CONFIRMATION_INPUT, COMPONENT_CONFIRMATION_RADIO, COMPONENT_INPUT_URL,
+    COMPONENT_LABEL_HELP, COMPONENT_PARAGRAPH_LYRIC, COMPONENT_PROGRESS,
+    COMPONENT_SEARCH_PLAYLIST_INPUT, COMPONENT_SEARCH_PLAYLIST_TABLE, COMPONENT_TABLE_QUEUE,
+    COMPONENT_TABLE_YOUTUBE, COMPONENT_TEXT_ERROR, COMPONENT_TEXT_HELP, COMPONENT_TREEVIEW,
 };
 use crate::{
     song::Song,
@@ -42,7 +42,7 @@ use crate::{
         MSG_KEY_CHAR_J, MSG_KEY_CHAR_K, MSG_KEY_CHAR_L, MSG_KEY_CHAR_MINUS, MSG_KEY_CHAR_N,
         MSG_KEY_CHAR_P, MSG_KEY_CHAR_PLUS, MSG_KEY_CHAR_R, MSG_KEY_CHAR_S, MSG_KEY_CHAR_T,
         MSG_KEY_CHAR_Y, MSG_KEY_CTRL_H, MSG_KEY_ENTER, MSG_KEY_ESC, MSG_KEY_SHIFT_TAB,
-        MSG_KEY_SPACE, MSG_KEY_TAB,
+        MSG_KEY_SLASH, MSG_KEY_SPACE, MSG_KEY_TAB,
     },
 };
 use humantime::format_duration;
@@ -51,10 +51,10 @@ use std::thread::{self, sleep};
 use std::time::Duration;
 use tui_realm_stdlib::{LabelPropsBuilder, ParagraphPropsBuilder, ProgressBarPropsBuilder};
 use tui_realm_treeview::TreeViewPropsBuilder;
+use tuirealm::props::Alignment;
 use tuirealm::{
     event::{Event, KeyCode, KeyEvent, KeyModifiers},
     props::TextSpan,
-    tui::layout::Alignment,
     Msg, Payload, PropsBuilder, Value,
 };
 
@@ -172,7 +172,7 @@ impl TermusicActivity {
                                self.mount_error(e.to_string().as_ref());
                            };
                         }
-                }
+                    }
                     None
                 },
 
@@ -241,18 +241,18 @@ impl TermusicActivity {
                 (COMPONENT_TREEVIEW,key) if key==  &MSG_KEY_CHAR_CAPITAL_L => {
                     // Add all songs in a folder to queue
                     if let Some(Payload::One(Value::Str(node_id))) = self.view.get_state(COMPONENT_TREEVIEW) {
-                            let p: &Path = Path::new(node_id.as_str());
-                            if p.is_dir() {
-                                let new_items = Self::dir_children(p);
-                                for i in new_items.iter().rev() {
-                                    match Song::from_str(i) {
-                                        Ok(s) => self.add_queue(s),
-                                        Err(e) => self.mount_error(format!("add queue error: {}",e).as_str()),
-                                    };
-                                }
+                        let p: &Path = Path::new(node_id.as_str());
+                        if p.is_dir() {
+                            let new_items = Self::dir_children(p);
+                            for i in new_items.iter().rev() {
+                                match Song::from_str(i) {
+                                    Ok(s) => self.add_queue(s),
+                                    Err(e) => self.mount_error(format!("add queue error: {}",e).as_str()),
+                                };
                             }
+                        }
                     }
-                            None
+                    None
                 }
 
                 (COMPONENT_TABLE_QUEUE,key) if key==  &MSG_KEY_CHAR_L => {
@@ -396,22 +396,6 @@ impl TermusicActivity {
                     self.umount_confirmation_input();
                     None
                 }
-                (_,key) if key==  &MSG_KEY_CHAR_H => {
-                    let event: Event = Event::Key(KeyEvent {
-                        code: KeyCode::Left,
-                        modifiers: KeyModifiers::NONE,
-                    });
-                    self.view.on(event);
-                    None
-                }
-                (_,key) if key==  &MSG_KEY_CHAR_L => {
-                    let event: Event = Event::Key(KeyEvent {
-                        code: KeyCode::Right,
-                        modifiers: KeyModifiers::NONE,
-                    });
-                    self.view.on(event);
-                    None
-                }
 
                 (COMPONENT_CONFIRMATION_RADIO, Msg::OnSubmit(_)) => {
                     if let Some(Payload::One(Value::Usize(index))) =
@@ -441,14 +425,14 @@ impl TermusicActivity {
                         }
                         if let Some(f) = song.lyric_frames.get(song.lyric_selected) {
                             if let Ok(parsed_lyric) = Lyric::from_str(&f.text) {
-                                let tx = self.sender_message.clone();
+                                let tx = self.sender.clone();
                                 song.parsed_lyric = Some(parsed_lyric);
                                 let lang_ext = f.description.clone();
                                 self.current_song = Some(song);
                                 thread::spawn(move || {
-                                    let _drop = tx.send(MessageState::Show(("Lyric switch successful".to_string(),format!("{} lyric is showing",lang_ext))));
+                                    let _drop = tx.send(UpdateComponents::MessageShow(("Lyric switch successful".to_string(),format!("{} lyric is showing",lang_ext))));
                                     sleep(Duration::from_secs(5));
-                                    let _drop = tx.send(MessageState::Hide);
+                                    let _drop = tx.send(UpdateComponents::MessageHide(("Lyric switch successful".to_string(),format!("{} lyric is showing",lang_ext))));
                                 });
                             }
                         }
@@ -494,6 +478,80 @@ impl TermusicActivity {
                     None
                 }
 
+                (COMPONENT_TREEVIEW,key) if key==  &MSG_KEY_SLASH=> {
+                    self.mount_search_playlist();
+                    // if let Err(e) = self.sync_search_playlist() {
+                    //    self.mount_error(format!("sync search playlist error: {}",e).as_str()); 
+                    // }
+                    self.init_search_playlist();
+                    None
+                }
+                (COMPONENT_SEARCH_PLAYLIST_INPUT,key) if (key==  &MSG_KEY_ESC) | (key == &MSG_KEY_CHAR_CAPITAL_Q)=> {
+                    self.umount_search_playlist();
+                    None
+                }
+
+                (COMPONENT_SEARCH_PLAYLIST_INPUT, Msg::OnChange(Payload::One(Value::Str(input)))) => {
+                    // Update span
+                    self.update_search_playlist(input);
+                    None
+                }
+
+                (COMPONENT_SEARCH_PLAYLIST_INPUT, Msg::OnSubmit(Payload::One(Value::Str(_)))) => {
+                    self.view.active(COMPONENT_SEARCH_PLAYLIST_TABLE);
+                    None
+                }
+
+                (COMPONENT_SEARCH_PLAYLIST_INPUT, key) if (key== &MSG_KEY_TAB) => {
+                    self.view.active(COMPONENT_SEARCH_PLAYLIST_TABLE);
+                    None
+                }
+
+                (COMPONENT_SEARCH_PLAYLIST_TABLE, key) if (key== &MSG_KEY_TAB) => {
+                    self.view.active(COMPONENT_SEARCH_PLAYLIST_INPUT);
+                    None
+                }
+
+                (COMPONENT_SEARCH_PLAYLIST_TABLE, key) if (key == &MSG_KEY_ENTER) => {
+                    if let Some(Payload::One(Value::Usize(index))) = self.view.get_state(COMPONENT_SEARCH_PLAYLIST_TABLE)
+                    {
+                        self.select_after_search_playlist(index);
+                    }
+                    self.umount_search_playlist();
+                    None
+                }
+
+                (COMPONENT_SEARCH_PLAYLIST_TABLE, key) if (key == &MSG_KEY_CHAR_L) => {
+                    if let Some(Payload::One(Value::Usize(index))) = self.view.get_state(COMPONENT_SEARCH_PLAYLIST_TABLE)
+                    {
+                        self.add_queue_after_search_playlist(index);
+                    }
+                    None
+                }
+
+
+                (COMPONENT_SEARCH_PLAYLIST_TABLE, key) if (key == &MSG_KEY_ESC) | (key == &MSG_KEY_CHAR_CAPITAL_Q) => {
+                    self.umount_search_playlist();
+                    None
+                }
+                         (_,key) if key==  &MSG_KEY_CHAR_H => {
+                    let event: Event = Event::Key(KeyEvent {
+                        code: KeyCode::Left,
+                        modifiers: KeyModifiers::NONE,
+                    });
+                    self.view.on(event);
+                    None
+                }
+
+                (_,key) if key==  &MSG_KEY_CHAR_L => {
+                    let event: Event = Event::Key(KeyEvent {
+                        code: KeyCode::Right,
+                        modifiers: KeyModifiers::NONE,
+                    });
+                    self.view.on(event);
+                    None
+                }
+
                 // Refresh playlist
                 (_,key) if key==  &MSG_KEY_CHAR_R => {
                     self.sync_playlist(None);
@@ -505,6 +563,8 @@ impl TermusicActivity {
                     self.exit_reason = Some(ExitReason::Quit);
                     None
                 }
+
+
                 _ => None,
             },
         }
@@ -558,22 +618,22 @@ impl TermusicActivity {
             None => return,
         };
 
-        if time_pos > self.time_pos || time_pos < 2 {
-            self.time_pos = time_pos;
-            if let Some(props) = self.view.get_props(COMPONENT_PROGRESS) {
-                let props = ProgressBarPropsBuilder::from(props)
-                    .with_progress(new_prog)
-                    .with_label(format!(
-                        "{}     :     {} ",
-                        format_duration(Duration::from_secs(time_pos as u64)),
-                        format_duration(Duration::from_secs(duration as u64))
-                    ))
-                    .build();
-                let msg = self.view.update(COMPONENT_PROGRESS, props);
-                self.redraw = true;
-                self.update(msg);
-            }
+        // if time_pos > self.time_pos || time_pos < 2 {
+        self.time_pos = time_pos;
+        if let Some(props) = self.view.get_props(COMPONENT_PROGRESS) {
+            let props = ProgressBarPropsBuilder::from(props)
+                .with_progress(new_prog)
+                .with_label(format!(
+                    "{}     :     {} ",
+                    format_duration(Duration::from_secs(time_pos as u64)),
+                    format_duration(Duration::from_secs(duration as u64))
+                ))
+                .build();
+            let msg = self.view.update(COMPONENT_PROGRESS, props);
+            self.redraw = true;
+            self.update(msg);
         }
+        // }
 
         // Update lyrics
         if self.queue_items.is_empty() {
@@ -655,30 +715,44 @@ impl TermusicActivity {
     }
 
     // change status bar text to indicate the downloading state
-    pub fn update_download_progress(&mut self) {
-        if let Ok(transfer_state) = self.receiver.try_recv() {
-            match transfer_state {
-                TransferState::Running => {
+    pub fn update_components(&mut self) {
+        if let Ok(update_components_state) = self.receiver.try_recv() {
+            match update_components_state {
+                UpdateComponents::DownloadRunning => {
                     self.update_status_line(StatusLine::Running);
                 }
-                TransferState::Success => {
+                UpdateComponents::DownloadSuccess => {
                     self.update_status_line(StatusLine::Success);
                 }
-                TransferState::Completed(Some(file)) => {
+                UpdateComponents::DownloadCompleted(Some(file)) => {
                     self.sync_playlist(Some(file.as_str()));
                     self.update_status_line(StatusLine::Default);
                 }
-                TransferState::Completed(None) => {
+                UpdateComponents::DownloadCompleted(None) => {
                     self.sync_playlist(None);
                     self.update_status_line(StatusLine::Default);
                 }
-                TransferState::ErrDownload => {
+                UpdateComponents::DownloadErrDownload => {
                     self.mount_error("download failed");
                     self.update_status_line(StatusLine::Error);
                 }
-                TransferState::ErrEmbedData => {
+                UpdateComponents::DownloadErrEmbedData => {
                     // This case will not happen in main activity
                 }
+                UpdateComponents::YoutubeSearchSuccess(y) => {
+                    self.youtube_options = y;
+                    self.sync_youtube_options();
+                    self.redraw = true;
+                }
+                UpdateComponents::YoutubeSearchFail(e) => {
+                    self.mount_error(&e);
+                }
+                UpdateComponents::MessageShow((title, text)) => {
+                    self.mount_message(&title, &text);
+                }
+                UpdateComponents::MessageHide((title, text)) => {
+                    self.umount_message(&title, &text);
+                }
             }
         };
     }
@@ -748,36 +822,6 @@ impl TermusicActivity {
         }
     }
 
-    // update message box
-    pub fn update_message(&mut self) {
-        if let Ok(message_state) = self.receiver_message.try_recv() {
-            match message_state {
-                MessageState::Show((title, text)) => {
-                    self.mount_message(&title, &text);
-                }
-                MessageState::Hide => {
-                    self.umount_message();
-                }
-            }
-        }
-    }
-
-    // update youtube search box
-    pub fn update_youtube_search(&mut self) {
-        if let Ok(youtube_search) = self.receiver_youtubesearch.try_recv() {
-            match youtube_search {
-                YoutubeSearchState::Success(y) => {
-                    self.youtube_options = y;
-                    self.sync_youtube_options();
-                    self.redraw = true;
-                }
-                YoutubeSearchState::Fail(e) => {
-                    self.mount_error(&e);
-                }
-            }
-        }
-    }
-
     // update queue items when loading
     pub fn update_queue_items(&mut self) {
         if let Ok(queue_items) = self.receiver_queueitems.try_recv() {
@@ -790,11 +834,17 @@ impl TermusicActivity {
     pub fn update_playing_song(&self) {
         if let Some(song) = &self.current_song {
             let name = song.name().unwrap_or("Unknown Song").to_string();
-            let tx = self.sender_message.clone();
+            let tx = self.sender.clone();
             thread::spawn(move || {
-                let _drop = tx.send(MessageState::Show(("Current Playing".to_string(), name)));
+                let _drop = tx.send(UpdateComponents::MessageShow((
+                    "Current Playing".to_string(),
+                    name.clone(),
+                )));
                 sleep(Duration::from_secs(5));
-                let _drop = tx.send(MessageState::Hide);
+                let _drop = tx.send(UpdateComponents::MessageHide((
+                    "Current Playing".to_string(),
+                    name,
+                )));
             });
         }
     }
