@@ -21,7 +21,7 @@
  * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  * SOFTWARE.
  */
-use super::{TermusicActivity, TransferState, COMPONENT_TABLE_YOUTUBE, COMPONENT_TREEVIEW};
+use super::{TermusicActivity, UpdateComponents, COMPONENT_TABLE_YOUTUBE, COMPONENT_TREEVIEW};
 use crate::invidious::{Instance, YoutubeVideo};
 use anyhow::{anyhow, bail, Result};
 use humantime::format_duration;
@@ -49,11 +49,6 @@ pub struct YoutubeOptions {
     invidious_instance: Instance,
 }
 
-pub enum YoutubeSearchState {
-    Success(YoutubeOptions),
-    Fail(String),
-}
-
 impl YoutubeOptions {
     pub fn new() -> Self {
         Self {
@@ -115,7 +110,7 @@ impl TermusicActivity {
 
     pub fn youtube_options_search(&mut self, keyword: &str) {
         let search_word = keyword.to_string();
-        let tx = self.sender_youtubesearch.clone();
+        let tx = self.sender.clone();
         thread::spawn(
             move || match crate::invidious::Instance::new(&search_word) {
                 Ok((instance, result)) => {
@@ -124,10 +119,10 @@ impl TermusicActivity {
                         page: 1,
                         invidious_instance: instance,
                     };
-                    let _drop = tx.send(YoutubeSearchState::Success(youtube_options));
+                    let _drop = tx.send(UpdateComponents::YoutubeSearchSuccess(youtube_options));
                 }
                 Err(e) => {
-                    let _drop = tx.send(YoutubeSearchState::Fail(e.to_string()));
+                    let _drop = tx.send(UpdateComponents::YoutubeSearchFail(e.to_string()));
                 }
             },
         );
@@ -198,6 +193,7 @@ impl TermusicActivity {
         }
     }
 
+    #[allow(clippy::too_many_lines)]
     pub fn youtube_dl(&mut self, link: &str) -> Result<()> {
         let mut path: PathBuf = PathBuf::new();
         if let Some(Payload::One(Value::Str(node_id))) = self.view.get_state(COMPONENT_TREEVIEW) {
@@ -226,7 +222,7 @@ impl TermusicActivity {
         let tx = self.sender.clone();
 
         thread::spawn(move || -> Result<()> {
-            let _drop = tx.send(TransferState::Running);
+            let _drop = tx.send(UpdateComponents::DownloadRunning);
             // start download
             let download = ytd.download();
 
@@ -257,22 +253,35 @@ impl TermusicActivity {
                                 let p = Path::new(&name);
                                 if let Some(ext) = p.extension() {
                                     if ext == "lrc" {
-                                        let mut lang_ext = "eng".to_string();
-                                        if let Some(p_short) = p.file_stem() {
-                                            let p2 = Path::new(p_short);
-                                            if let Some(ext2) = p2.extension() {
-                                                lang_ext = ext2.to_string_lossy().to_string();
+                                        if let Some(stem_lrc) = p.file_stem() {
+                                            let p1: &Path = Path::new(&file_fullname);
+                                            if let Some(p_base) = p1.file_stem() {
+                                                if stem_lrc
+                                                    .to_string_lossy()
+                                                    .to_string()
+                                                    .contains(p_base.to_string_lossy().as_ref())
+                                                {
+                                                    let mut lang_ext = "eng".to_string();
+                                                    if let Some(p_short) = p.file_stem() {
+                                                        let p2 = Path::new(p_short);
+                                                        if let Some(ext2) = p2.extension() {
+                                                            lang_ext =
+                                                                ext2.to_string_lossy().to_string();
+                                                        }
+                                                    }
+                                                    let lyric_string =
+                                                        std::fs::read_to_string(f.path());
+                                                    id3_tag.add_lyrics(Lyrics {
+                                                        lang: "eng".to_string(),
+                                                        description: lang_ext,
+                                                        text: lyric_string.unwrap_or_else(|_| {
+                                                            String::from("[00:00:01] No lyric")
+                                                        }),
+                                                    });
+                                                    let _drop = std::fs::remove_file(f.path());
+                                                }
                                             }
                                         }
-                                        let lyric_string = std::fs::read_to_string(f.path());
-                                        id3_tag.add_lyrics(Lyrics {
-                                            lang: "eng".to_string(),
-                                            description: lang_ext,
-                                            text: lyric_string.unwrap_or_else(|_| {
-                                                String::from("[00:00:01] No lyric")
-                                            }),
-                                        });
-                                        let _drop = std::fs::remove_file(f.path());
                                     }
                                 }
                             }
@@ -280,20 +289,21 @@ impl TermusicActivity {
 
                         let _drop = id3_tag.write_to_path(&file_fullname, id3::Version::Id3v24);
 
-                        let _drop = tx.send(TransferState::Success);
+                        let _drop = tx.send(UpdateComponents::DownloadSuccess);
                         sleep(Duration::from_secs(5));
-                        let _drop = tx.send(TransferState::Completed(Some(file_fullname)));
+                        let _drop =
+                            tx.send(UpdateComponents::DownloadCompleted(Some(file_fullname)));
                     } else {
                         // This shoudn't happen unless the output format of youtubedl changed
-                        let _drop = tx.send(TransferState::Success);
+                        let _drop = tx.send(UpdateComponents::DownloadSuccess);
                         sleep(Duration::from_secs(5));
-                        let _drop = tx.send(TransferState::Completed(None));
+                        let _drop = tx.send(UpdateComponents::DownloadCompleted(None));
                     }
                 }
                 ResultType::IOERROR | ResultType::FAILURE => {
-                    let _drop = tx.send(TransferState::ErrDownload);
+                    let _drop = tx.send(UpdateComponents::DownloadErrDownload);
                     sleep(Duration::from_secs(5));
-                    let _drop = tx.send(TransferState::Completed(None));
+                    let _drop = tx.send(UpdateComponents::DownloadCompleted(None));
                 }
             }
             Ok(())
